#### 目录介绍
- 01.创建型模式
- 02.包含模式
- 03.解决什么问题
- 04.简单介绍


### 01.创建型模式
- 创建型模式\(Creational Pattern\)对类的实例化过程进行了抽象，能够将软件模块中对象的创建和对象的使用分离。
- 为了使软件的结构更加清晰，外界对于这些对象只需要知道它们共同的接口，而不清楚其具体的实现细节，使整个系统的设计更加符合单一职责原则。
- 创建型模式在创建什么\(What\)，由谁创建\(Who\)，何时创建\(When\)等方面都为软件设计者提供了尽可能大的灵活性。创建型模式隐藏了类的实例的创建细节，通过隐藏对象如何被创建和组合在一起达到使整个系统独立的目的。



### 02.包含模式
- 分类
    * **简单工厂模式（Simple Factory）**
    * **工厂方法模式（Factory Method）**
    * **抽象工厂模式（Abstract Factory）**
    * **建造者模式（Builder）**
    * **原型模式（Prototype）**
    * **单例模式（Singleton）**
- 常见的有
    - 单例模式
    - 工厂模式
    - 建造者模式



### 03.解决什么问题
- 主要解决对象的创建问题，封装复杂的创建过程，解耦对象的创建代码和使用代码。


### 04.简单介绍
#### 4.1 单例模式
- 单例模式用来创建全局唯一的对象。一个类只允许创建一个对象（或者叫实例），那这个类就是一个单例类，这种设计模式就叫作单例模式。单例有几种经典的实现方式，它们分别是：饿汉式、懒汉式、双重检测、静态内部类、枚举。
- 尽管单例是一个很常用的设计模式，在实际的开发中，我们也确实经常用到它，但是，有些人认为单例是一种反模式（anti-pattern），并不推荐使用，主要的理由有以下几点：
    - 单例对 OOP 特性的支持不友好
    - 单例会隐藏类之间的依赖关系
    - 单例对代码的扩展性不友好
    - 单例对代码的可测试性不友好
    - 单例不支持有参数的构造函数


#### 4.2 工厂模式
- 工厂模式用来创建不同但是相关类型的对象（继承同一父类或者接口的一组子类），由给定的参数来决定创建哪种类型的对象。实际上，如果创建对象的逻辑并不复杂，那我们直接通过 new 来创建对象就可以了，不需要使用工厂模式。当创建逻辑比较复杂，是一个“大工程”的时候，我们就考虑使用工厂模式，封装对象的创建过程，将对象的创建和使用相分离。
- 当每个对象的创建逻辑都比较简单的时候，我推荐使用简单工厂模式，将多个对象的创建逻辑放到一个工厂类中。当每个对象的创建逻辑都比较复杂的时候，为了避免设计一个过于庞大的工厂类，我们推荐使用工厂方法模式，将创建逻辑拆分得更细，每个对象的创建逻辑独立到各自的工厂类中。
- 详细点说，工厂模式的作用有下面4个，这也是判断要不要使用工厂模式最本质的参考标准。
    - 封装变化：创建逻辑有可能变化，封装成工厂类之后，创建逻辑的变更对调用者透明。
    - 代码复用：创建代码抽离到独立的工厂类之后可以复用。
    - 隔离复杂性：封装复杂的创建逻辑，调用者无需了解如何创建对象。
    - 控制复杂度：将创建代码抽离出来，让原本的函数或类职责更单一，代码更简洁。



#### 4.3 建造者模式
- 建造者模式用来创建复杂对象，可以通过设置不同的可选参数，“定制化”地创建不同的对象。建造者模式的原理和实现比较简单，重点是掌握应用场景，避免过度使用。
- 如果一个类中有很多属性，为了避免构造函数的参数列表过长，影响代码的可读性和易用性，我们可以通过构造函数配合set()方法来解决。但是，如果存在下面情况中的任意一种，我们就要考虑使用建造者模式了。
    - 我们把类的必填属性放到构造函数中，强制创建对象的时候就设置。如果必填的属性有很多，把这些必填属性都放到构造函数中设置，那构造函数就又会出现参数列表很长的问题。如果我们把必填属性通过 set() 方法设置，那校验这些必填属性是否已经填写的逻辑就无处安放了。
    - 如果类的属性之间有一定的依赖关系或者约束条件，我们继续使用构造函数配合 set() 方法的设计思路，那这些依赖关系或约束条件的校验逻辑就无处安放了。
    - 如果我们希望创建不可变对象，也就是说，对象在创建好之后，就不能再修改内部的属性值，要实现这个功能，我们就不能在类中暴露 set() 方法。构造函数配合 set() 方法来设置属性值的方式就不适用了。



#### 4.4 原型模式
- 如果对象的创建成本比较大，而同一个类的不同对象之间差别不大（大部分字段都相同），在这种情况下，我们可以利用对已有对象（原型）进行复制（或者叫拷贝）的方式，来创建新对象，以达到节省创建时间的目的。这种基于原型来创建对象的方式就叫作原型模式。















