# 11.视频播放器音频焦点抢占
#### 目录介绍
- 01.先说一下音视频焦点问题
- 02.为何处理音视频焦点抢占
- 03.处理音频焦点思路步骤
- 04.何时才会失去音视频焦点
- 05.请求和放弃音频焦点
- 06.音视频焦点变化处理逻辑



### 00.视频播放器通用框架
- 基础封装视频播放器player，可以在ExoPlayer、MediaPlayer，声网RTC视频播放器内核，原生MediaPlayer可以自由切换
- 对于视图状态切换和后期维护拓展，避免功能和业务出现耦合。比如需要支持播放器UI高度定制，而不是该lib库中UI代码
- 针对视频播放，音频播放，播放回放，以及视频直播的功能。使用简单，代码拓展性强，封装性好，主要是和业务彻底解耦，暴露接口监听给开发者处理业务具体逻辑
- 该播放器整体架构：播放器内核(自由切换) +  视频播放器 + 边播边缓存 + 高度定制播放器UI视图层
- 项目地址：https://github.com/yangchong211/YCVideoPlayer
- 关于视频播放器整体功能介绍文档：https://juejin.im/post/6883457444752654343




### 01.先说一下音视频焦点问题
- 如果手机上安装了两个音频播放器，当一个正在播放的时候，打开第二个播放歌曲，有没有发现第一个自动暂停了……
- 如果你在听音频的同时，又去打开了其它视频APP，你会发现音频APP暂停播放了……
- 如果你正在听音频或者看视频时，来电话了，那么音视频便会暂停。挂了电话后音乐又继续播放，视频则需要点击按钮播放，是不是很奇怪
- 当你收到消息，比如微信消息，并且有消息声音的时候，那么听音频的那一瞬间，音频的声音会变小了，然后过会儿又恢复了。是不是很有意思。
- 别蒙圈，这个就叫做音频捕获和丢弃焦点。



### 02.为何处理音视频焦点抢占
- 如果不处理捕获与丢弃音频焦点的话，那么同时开几个音视频播放器，就会出现多个声音。那样会很嘈杂，一般线上的APP都会做这个处理，不过一些GitHub案例demo中一般没处理。
- 为了协调设备的音频输出，android提出了Audio Focus机机制，获取audio focus必须调用AudioManager的requestAudioFocus()方法。


### 03.处理音频焦点思路步骤
- 简单来说，就是这三步逻辑方法
    - 在onCreate方法中调用初始化方法
    - 在播放音视频的时候开始请求捕获音频焦点
    - 在音视频销毁的时候开始丢弃音频焦点


### 04.何时才会失去音视频焦点
- 失去焦点有三种类型
    - 1.失去短暂焦点
    - 2.失去永久焦点
    - 3.Ducking
- 失去焦点原理说明
    - 当重新获得焦点的时候，如果通话结束，恢复播放；获取音量并且恢复音量。这个情景应该经常遇到。
    - 当永久丢失焦点，比如同时打开播放器，则停止或者暂停播放，否则出现两个声音
    - 当短暂丢失焦点，比如比如来了电话或者微信视频音频聊天等等，则暂停或者停止播放
    - 当瞬间丢失焦点，比如手机来了通知。前提是你的通知是震动或者声音时，会短暂地将音量减小一半。当然你也可以减小三分之一，哈哈！


### 05.请求和放弃音频焦点
- 首先获取AudioManager对象
    ``` java
    mAudioManager = (AudioManager) content.getSystemService(AUDIO_SERVICE);
    ```
- 请求和放弃音频焦点
    - AudioFocus这个其实是音频焦点，一般情况下音视频播放器都会处理这个音频焦点的，在其丢失音频焦点的情况会将音频暂停或者停止的逻辑的，等到再次获取到音频焦点的情况下会再次恢复播放的。
    - 音频获取焦点可以通过requestAudioFocus()方法获得，在音频焦点成功获取后，该方法会返回AUDIOFOCUS_REQUEST_GRANTED常量，否则，会返回AUDIOFOCUS_REQUEST_FAILED常量。
    - 音视频失去焦点abandonAudioFocus()方法，这会通知系统您的App不再需要音频焦点，并移除相关OnAudioFocusChangeListener的注册。如果释放的是短暂音调焦点，那么被打断的音频会被继续播放。
- **代码如下所示**，下面是简单的代码。具体看：AudioFocusHelper类
    ``` java
    /**
     * 请求音频焦点，开始播放时候调用
     * @return
     */
    public boolean requestAudioFocus() {
        return mAudioManager.requestAudioFocus(this, AudioManager.STREAM_MUSIC,
                AudioManager.AUDIOFOCUS_GAIN) == AudioManager.AUDIOFOCUS_REQUEST_GRANTED;
    }
    
    /**
     * 放弃音频焦点，销毁播放时候调用
     */
    public void abandonAudioFocus() {
        mAudioManager.abandonAudioFocus(this);
    }
    ```

### 06.音视频焦点变化处理逻辑
- 当焦点发生变化的时候，可以在这个方法onAudioFocusChange中处理业务逻辑，由于onAudioFocusChange有可能在子线程调用，所以需要切换线程处理逻辑
    ``` java
    @Override
    public void onAudioFocusChange(final int focusChange) {
        if (mCurrentFocus == focusChange) {
            return;
        }
        //由于onAudioFocusChange有可能在子线程调用，
        //故通过此方式切换到主线程去执行
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                //处理音频焦点抢占
                handleAudioFocusChange(focusChange);
            }
        });
        mCurrentFocus = focusChange;
    }
    ```
- 当音频焦点发生变化的时候调用这个方法，在这里可以处理逻辑。详细案例，可以直接参考我的demo
    ``` java
    private void handleAudioFocusChange(int focusChange) {
        final VideoPlayer videoView = mWeakVideoView.get();
        if (videoView == null) {
            return;
        }
        switch (focusChange) {
            case AudioManager.AUDIOFOCUS_GAIN:
                //重新获得焦点
            case AudioManager.AUDIOFOCUS_GAIN_TRANSIENT:
                //暂时获得焦点
                if (mStartRequested || mPausedForLoss) {
                    videoView.start();
                    mStartRequested = false;
                    mPausedForLoss = false;
                }
                if (!videoView.isMute())
                    //恢复音量
                    videoView.setVolume(1.0f, 1.0f);
                break;
            case AudioManager.AUDIOFOCUS_LOSS:
                //焦点丢失，这个是永久丢失焦点，如被其他播放器抢占
            case AudioManager.AUDIOFOCUS_LOSS_TRANSIENT:
                //焦点暂时丢失，，如来电
                if (videoView.isPlaying()) {
                    mPausedForLoss = true;
                    videoView.pause();
                }
                break;
            case AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK:
                //此时需降低音量，瞬间丢失焦点，如通知
                if (videoView.isPlaying() && !videoView.isMute()) {
                    videoView.setVolume(0.1f, 0.1f);
                }
                break;
        }
    }
    ```
- focusChange参数值
    - 1.AUDIOFOCUS_GAIN：获取audio focus
    - 2.AUDIOFOCUS_LOSS：失去audio focus很长一段时间，必须停止所有的audio播放，清理资源
    - 3.AUDIOFOCUS_ LOSS_TRANSIENT：暂时失去audio focus，但是很快就会重新获得，在此状态应该暂停所有音频播放，但是不能清除资源
    - 4.AUDIOFOCUS_ LOSS_TRANSIENT _CAN_DUCK：暂时失去 audio focus，但是允许持续播放音频(以很小的声音)，不需要完全停止播放。























